#ifndef DOORKEEPER_DEV_DISPLAY_MANAGER_H
#define DOORKEEPER_DEV_DISPLAY_MANAGER_H

#include <atomic>
#include <memory>
#include <string>

#include <kbf.h>
#include <kbf/driver/lcd.h>
#include <kbf/rtos.h>
#include <kbf/task.h>

class DisplayManager : public kbf::Task {
public:
    static constexpr const char *const TAG = "DisplayManager";

    static const int defaultTimeout;

    DisplayManager(const std::string &name, void *arg, uint32_t stackSize, uint32_t priority);

    static void start();

    enum class WifiState {
        AP,
        ONLINE,
        OFFLINE,
        CONNECTING,
    };

    static void setWifiState(WifiState state);

    enum class DoorState {
        NOT_CALIBRATED,
        OPEN,
        CLOSED,
    };

    static void setDoorState(DoorState state);

    static void setStatus(std::string text);

    static void setMessage(std::string text, int timeoutMs = 0);

protected:
    [[noreturn]] void run(void *arg) override;

private:
    static std::shared_ptr<kbf::driver::LCD> lcd;

    static std::atomic<WifiState> wifiState;
    static std::atomic<DoorState> doorState;
    static std::string            status;
    static std::string            message;

    static std::shared_ptr<kbf::rtos::EventGroup> eventGroup;
    struct Events {
        static const constexpr unsigned int WIFI    = 0;
        static const constexpr unsigned int DOOR    = 1;
        static const constexpr unsigned int STATUS  = 2;
        static const constexpr unsigned int MESSAGE = 3;
    };

    const std::vector<int> ALL_EVENTS = {
            Events::WIFI,
            Events::DOOR,
            Events::STATUS,
            Events::MESSAGE,
    };

    class MessageTimeout : public kbf::Task {
    public:
        MessageTimeout(const std::string &name, void *arg, uint32_t stackSize, uint32_t priority) :
                Task(name, arg, stackSize, priority) {}

    protected:
        void run(void *arg) override;
    };

    static std::shared_ptr<kbf::Task> messageTimeout;

    static std::atomic<int> timeoutMs_;
};

#endif //DOORKEEPER_DEV_DISPLAY_MANAGER_H
