#ifndef DOORKEEPER_DEV_EVENT_MANAGER_H
#define DOORKEEPER_DEV_EVENT_MANAGER_H

#include <optional>
#include <string>

#include <kbf/http/client.h>
#include <kbf/queue.h>
#include <kbf/rtos.h>
#include <kbf/task.h>

#define API_URL CONFIG_DK_SERVER_API_URL "/dev/event"

class EventManager : public kbf::Task {
public:
    static constexpr const char *const TAG = "EventManager";

    EventManager(const std::string &name, void *arg, uint32_t stackSize, uint32_t priority) :
            Task(name, arg, stackSize, priority) {}

    static void start();

    static void stop();

    enum class Event {
        BOOT = 0,
        NOOP = 1,
        DOOR_OPEN = 2,
        DOOR_CLOSED = 3,
    };

    static bool event(Event event);

protected:
    [[noreturn]] void run(void *arg) override;

private:
    std::string pairingId;

    static kbf::Queue<Event> eventQueue;
    std::optional<Event>     lastEvent;

    bool postEvent(Event event);

    unsigned int noopInterval = CONFIG_DK_DEFAULT_NOOP_INTERVAL;
};

#endif //DOORKEEPER_DEV_EVENT_MANAGER_H
