#include "dk.h"

#include <vector>
#include <atomic>

#include <kbf.h>
#include <kbf/wifi.h>
#include <kbf/http/client.h>

#include "settings.h"
#include "tasks/display_manager.h"
#include "tasks/event_manager.h"
#include "tasks/ultrasonic.h"


using namespace kbf;
using std::vector;
using std::atomic;
using std::shared_ptr;

namespace dk {
    shared_ptr<driver::Button> button;

    shared_ptr<rtos::EventGroup> mainEventGroup;

    std::shared_ptr<kbf::WebService> pairingService;
    std::shared_ptr<kbf::WebService> apiService;

    void setPairing(const string &pairingId) {
        ESP_LOGI(TAG, "%s(%s)", __func__, pairingId.c_str());
        dk::settings::setPairingId(pairingId);
        EventManager::start();
        Ultrasonic::enable();
        mainEventGroup->setBit(MainEvents::SET_ONLINE_MODE);
    }

    void clearPairing() {
        ESP_LOGI(TAG, "%s", __func__);
        Ultrasonic::disable();
        dk::settings::setPairingId("");
        wifi::stop();
        mainEventGroup->setBit(MainEvents::SET_PAIRING_MODE);
        EventManager::stop();
    }

    bool paired() {
        return !dk::settings::pairingId().empty();
    }

    // old stuff starts here

    void testNotification() {
        ESP_LOGI(TAG, "testNotification()");

        if (!paired()) {
            ESP_LOGI(TAG, "  not paired");
            DisplayManager::setMessage("not paired", DisplayManager::defaultTimeout);
            return;
        }

        auto pairingId = dk::settings::pairingId();
        ESP_LOGI(TAG, "  pairingId = %s", pairingId.c_str());
        auto client   = http::Client(CONFIG_DK_HTTP_CLIENT_TIMEOUT);
        auto response = client.get(CONFIG_DK_SERVER_API_URL "/dev/test_notification",
                                    {{{"X-Pairing-ID", pairingId}}});
        ESP_LOGI(TAG, "test_notification response: %d", response->status);

        DisplayManager::setMessage("message sent", DisplayManager::defaultTimeout);
    }
}
