#include <kbf/wifi.h>
#include <kbf/web_service.h>
#include <kbf/http/client.h>

#include "settings.h"
#include "api/calibrate_controller.h"
#include "tasks/display_manager.h"
#include "tasks/ultrasonic.h"

#include <unity.h>
#include <kbf.h>

using namespace kbf;

#define TEST_URL "http://localhost/calibrate"

TEST_CASE("CalibrateController", "[controller]") {
    // TODO mock these maybe so the test passes without a real device (e.g. test dev)
    DisplayManager::start();
    Ultrasonic::start();

    dk::settings::Calibration config = {0, 0};
    dk::settings::setCalibration(config);

    wifi::sta::start();

    auto webService = WebService();
    webService.controller<CalibrateController>();
    webService.start();

    auto client = http::Client();

    auto response = client.get(TEST_URL);
    TEST_ASSERT_EQUAL(200, response->status);
    auto json = response->json();
    TEST_ASSERT_EQUAL(0, json.find("high")->get<int>());
    TEST_ASSERT_EQUAL(0, json.find("low")->get<int>());

    response = client.post(TEST_URL, {});
    TEST_ASSERT_EQUAL(200, response->status);
    json = response->json();
    const int high = json.find("high")->get<int>();
    const int low  = json.find("low")->get<int>();
    TEST_ASSERT_NOT_EQUAL(0, high);
    TEST_ASSERT_NOT_EQUAL(0, low);

    config = dk::settings::calibration();
    TEST_ASSERT_EQUAL(high, config.high);
    TEST_ASSERT_EQUAL(low, config.low);

    response = client.get(TEST_URL);
    TEST_ASSERT_EQUAL(200, response->status);
    json = response->json();
    TEST_ASSERT_EQUAL(high, json.find("high")->get<int>());
    TEST_ASSERT_EQUAL(low, json.find("low")->get<int>());

    kbf::sleep(1000);
    webService.stop();
    wifi::stop();
}