#include <kbf/wifi.h>
#include <kbf/http/client.h>
#include <kbf/web_service.h>

#include <unity.h>
#include <settings.h>
#include <api/pairing_id_middleware.h>

using namespace kbf;
using namespace std;

static const constexpr char *const testUrl        = "http://localhost/test";
static const constexpr char *const testPairingId  = "1647daae-d4cf-4023-82e7-da57cb14106c";
static const constexpr char *const wrongPairingId = "b2004747-d874-4094-aa8e-abf424c2939c";

class MiddlewareTestController : public WebService::Controller {
public:
    MiddlewareTestController() : Controller("/test") {}

protected:
    http::Response get(const http::Request &request) override {
        return http::Response("OK");
    }
};

TEST_CASE("PairingIdMiddleware", "[middleware]") {
    wifi::sta::start();
    dk::settings::setPairingId(testPairingId);

    WebService webService;
    webService.controller<MiddlewareTestController>();
    webService.middleware<PairingIdMiddleware>();
    webService.start();

    auto client = http::Client();

    auto response = client.get(testUrl);
    TEST_ASSERT_EQUAL(403, response->status);

    response = client.get(testUrl, {{{"X-Pairing-ID", wrongPairingId}}});
    TEST_ASSERT_EQUAL(403, response->status);

    response = client.get(testUrl, {{{"X-Pairing-ID", testPairingId}}});
    TEST_ASSERT_EQUAL(200, response->status);

    webService.stop();
    wifi::stop();
}