#include <iostream>

#include <nlohmann_json/json.hpp>
#include <kbf.h>
#include <kbf/rtos.h>
#include <kbf/wifi.h>
#include <kbf/http/client.h>
#include <kbf/web_service.h>

#include "pairing_service/scan_controller.h"

#include <unity.h>

using namespace kbf;
using namespace std;

#define DK_TEST_SSID "scan_controller_test"
#define DK_TEST_PASS "scan_controller_pass"

rtos::EventGroup eventGroup;

void scanControllerMaster() {
    wifi::ap::onDisconnect = {[](wifi::STA &) { eventGroup.setBit(0); }};
    wifi::ap::start(DK_TEST_SSID, DK_TEST_PASS);

    eventGroup.waitForBit(0);
    wifi::stop();
}

void scanControllerSlave() {
    wifi::sta::start();

    auto webService = WebService();
    webService.controller<ScanController>();
    webService.start();

    auto client   = http::Client();

    for (int i = 0; i < 30; i++) {
        cout << "cycle " << i << " / 30" << endl;
        auto response = client.get("http://localhost/scan");
        cout << response->body << endl;
        auto responseJson = nlohmann::json::parse(response->body);
        if (responseJson["finished"]) {
            bool found = false;
            for (const auto &ap : responseJson["results"]) {
                if (ap["ssid"] == DK_TEST_SSID) {
                    found = true;
                    break;
                }
            }
            TEST_ASSERT_TRUE(found)
            break;
        }
        kbf::sleep(200);
    }

    TEST_ASSERT_TRUE(wifi::sta::connect(DK_TEST_SSID, DK_TEST_PASS));

    webService.stop();
    wifi::stop();
}

TEST_CASE_MULTIPLE_DEVICES("ScanController", "[controller]", scanControllerMaster, scanControllerSlave)
