#include <atomic>

#include <kbf.h>
#include <kbf/nvs.h>
#include <kbf/rtos.h>
#include <kbf/wifi.h>
#include <kbf/http/client.h>
#include <kbf/web_service.h>

#include "pairing_service/settings_controller.h"

#include <unity.h>

using namespace kbf;
using namespace std;

static const char *const url          = "http://192.168.4.1/settings";
static const char *const masterSSID   = "doorkeeper_test_master";
static const char *const slaveSSID    = "doorkeeper_test_slave";
static const char *const testPassword = "mg4drg9ss9e8t";

static rtos::EventGroup eventGroup;

void testSettingsControllerMaster() {
    nvs::erase();

    wifi::ap::onDisconnect = {[](wifi::STA &) { eventGroup.setBit(0); }};
    wifi::dual::start(masterSSID, testPassword);

    auto webService = WebService();
    webService.controller<SettingsController>();
    webService.start();

    eventGroup.waitForBit(0);

    webService.stop();
    wifi::stop();
}

static atomic<bool> masterConnected;

void testSettingsControllerSlave() {
    masterConnected     = false;
    wifi::ap::onConnect = {[](wifi::STA &) { masterConnected = true; }};

    wifi::dual::start(slaveSSID, testPassword, net::IP("192.168.1.1"));
    if (!wifi::sta::connect(masterSSID, testPassword)) TEST_FAIL();


    auto client = http::Client(13000);

    auto responseJson = client.get(url)->json();
    TEST_ASSERT_TRUE(responseJson.find("ssid")->get<string>().empty())
    TEST_ASSERT_TRUE(responseJson.find("password")->get<string>().empty())

    auto response = client.post(url, {
        {"ssid",     slaveSSID},
        {"password", "wrongPassword"}
    });
    TEST_ASSERT_EQUAL(200, response->status);
    TEST_ASSERT_FALSE(response->json().find("success")->get<bool>())

    TEST_ASSERT_FALSE(masterConnected)
    response = client.post(url, {
        {"ssid",     slaveSSID},
        {"password", testPassword}
    });
    TEST_ASSERT_EQUAL(200, response->status);
    TEST_ASSERT_TRUE(response->json().find("success")->get<bool>())
    TEST_ASSERT_TRUE(masterConnected)

    responseJson = client.get(url)->json();
    TEST_ASSERT_EQUAL_STRING(slaveSSID, responseJson.find("ssid")->get<string>().c_str());
    TEST_ASSERT_EQUAL_STRING(testPassword, responseJson.find("password")->get<string>().c_str());

    response = client.post(url, {
        {"ssid", masterSSID},
    });
    TEST_ASSERT_EQUAL(500, response->status);

    response = client.post(url, {
        {"password", testPassword},
    });
    TEST_ASSERT_EQUAL(500, response->status);

    response = client.post(url, {});
    TEST_ASSERT_EQUAL(500, response->status);

    wifi::stop();
}

TEST_CASE_MULTIPLE_DEVICES("SettingsController", "[controller_multi]",
                           testSettingsControllerMaster, testSettingsControllerSlave)