#include <unity.h>

#include <iostream>

#include <kbf.h>
#include <kbf/nvs.h>
#include <esp_system.h>

#include "settings.h"

using namespace kbf;
using namespace std;

#define TEST_SSID "doorkeeper_test_ssid"
#define TEST_PASSWORD "doorkeeper_test_pass"
#define TEST_PAIRING_ID "71023842-5b39-4e62-957d-0a7345f5fd8e"
#define TEST_CALIB_LOW 89
#define TEST_CALIB_HIGH 90

void stage1() {
    nvs::init();
    nvs::erase();

    dk::settings::init();

    auto wifiConfig = dk::settings::wifi();
    TEST_ASSERT_EQUAL_STRING("", wifiConfig.ssid.c_str());
    TEST_ASSERT_EQUAL_STRING("", wifiConfig.password.c_str());
    wifiConfig = {TEST_SSID, TEST_PASSWORD};
    dk::settings::setWifi(wifiConfig);

    auto pairingId = dk::settings::pairingId();
    TEST_ASSERT_EQUAL_STRING("", pairingId.c_str());
    dk::settings::setPairingId(TEST_PAIRING_ID);

    auto calibration = dk::settings::calibration();
    TEST_ASSERT_EQUAL(0, calibration.high);
    TEST_ASSERT_EQUAL(0, calibration.low);
    calibration = {TEST_CALIB_HIGH, TEST_CALIB_LOW};
    dk::settings::setCalibration(calibration);

    cout << "restarting in 1..." << endl;
    kbf::sleep(1000);
    esp_restart();
}

void stage2() {
    auto wifiConfig = dk::settings::wifi();
    TEST_ASSERT_EQUAL_STRING(TEST_SSID, wifiConfig.ssid.c_str());
    TEST_ASSERT_EQUAL_STRING(TEST_PASSWORD, wifiConfig.password.c_str());

    auto pairingId = dk::settings::pairingId();
    TEST_ASSERT_EQUAL_STRING(TEST_PAIRING_ID, pairingId.c_str());

    auto calibration = dk::settings::calibration();
    TEST_ASSERT_EQUAL(TEST_CALIB_HIGH, calibration.high);
    TEST_ASSERT_EQUAL(TEST_CALIB_LOW, calibration.low);

    nvs::erase();
}

TEST_CASE_MULTIPLE_STAGES("SettingsManager", "[settings_manager]", stage1, stage2)
