#include <kbf.h>
#include <kbf/nvs.h>
#include <kbf/rtos.h>
#include <kbf/wifi.h>
#include <kbf/http/client.h>
#include <kbf/web_service.h>

#include "api/unpair_controller.h"
#include "settings.h"

#include <unity.h>

using namespace kbf;
using namespace std;

static const char *const url           = "http://192.168.4.1/unpair";
static const char *const ssid          = "doorkeeper_test_master";
static const char *const testPassword  = "mg4drg9ss9e8t";
static const char *const testPairingId = "1647daae-d4cf-4023-82e7-da57cb14106c";

static rtos::EventGroup eventGroup;

void unpairControllerMaster() {
    nvs::erase();
    dk::settings::setPairingId(testPairingId);

    wifi::ap::onDisconnect = {[](wifi::STA &) { eventGroup.setBit(0); }};
    wifi::ap::start(ssid, testPassword);

    auto webService = WebService();
    webService.controller<UnpairController>();
    webService.start();

    eventGroup.waitForBit(0);

    TEST_ASSERT_EQUAL_STRING("", dk::settings::pairingId().c_str());

    kbf::sleep(500);
    webService.stop();
    wifi::stop();
}

void unpairControllerSlave() {
    wifi::sta::start();
    TEST_ASSERT_TRUE(wifi::sta::connect(ssid, testPassword));

    auto client   = http::Client();
    auto response = client.get(url);
    TEST_ASSERT_EQUAL(200, response->status);

    kbf::sleep(500);
    wifi::stop();
}

TEST_CASE_MULTIPLE_DEVICES("UnpairController", "[controller]", unpairControllerMaster, unpairControllerSlave)
