#!/usr/bin/env bash

API_KEY_FILE=.api_key
DEVICE_ID_FILE=device_id

if [ "${1}" = "-v" ]; then
  curl_opts="-v"
else
  curl_opts="-sS"
fi

server_url=$(sed -E 's/^CONFIG_DK_SERVER_API_URL="(.*)"/\1/g;t;d' sdkconfig)

if [ -z "${server_url}" ]; then
  echo "CONFIG_DK_SERVER_API_URL not found, please run 'idf.py menuconfig'"
  exit 1
else
  echo "using server ${server_url}"
fi

if [ ! -f "${API_KEY_FILE}" ]; then
  echo "${API_KEY_FILE} not found"
  read -r -p "enter API key: " api_key
  echo "${api_key}" > "${API_KEY_FILE}"
  chmod 600 "${API_KEY_FILE}"
  echo "saved to ${API_KEY_FILE}"
fi

api_key=$(cat ${API_KEY_FILE})
echo "using API key \"${api_key}\""

device_id=$(curl "${server_url}/register/device" -d '{"api_key": "'"${api_key}"'"}' "${curl_opts}" | jq '.uuid' -r)
if [ -z "${device_id}" ]; then
  echo "request failed; try \"-v\" for details"
  exit 2
fi

echo "generated device ID \"${device_id}\""
echo -n "${device_id}" > "${DEVICE_ID_FILE}"
